@@ -1,9 +1,11 @@
 import logging
+import time
 
 API_BASE = "https://{}/api/v2"
 
 _LOGGER = logging.getLogger(__name__)
 
+# Valid clouds for this module
 clouds = {
     "worx": {
         "url": "api.worxlandroid.com",
@@ -21,17 +23,31 @@ clouds = {
 
 
 class WorxLandroidAPI:
-    def __init__(self):
+    """Worx Cloud API definition."""
+
+    def __init__(self) -> None:
+        """Initialize new instance of API broker."""
         self.cloud = clouds["worx"]
         self._token_type = "app"
+        self.username = None
+        self.password = None
+        self.type = None
+        self._tokenrefresh = 0
 
-    def set_token(self, token):
+    def set_token(self, token: str, now: int = -1) -> None:
+        """Set the token used for communication."""
+        if now == -1:
+            now = int(time.time())
+
+        self._tokenrefresh = now
         self._token = token
 
-    def set_token_type(self, token_type):
+    def set_token_type(self, token_type: str) -> None:
+        """Set type of token."""
         self._token_type = token_type
 
-    def _generate_identify_token(self, seed_token):
+    def _generate_identify_token(self, seed_token: str) -> str:
+        """Generate identify token."""
         text_to_char = [ord(c) for c in self.cloud["url"]]
 
         import re
@@ -57,63 +73,85 @@ class WorxLandroidAPI:
         final = "".join(step_four)
         return final
 
-    def _get_headers(self):
+    def _get_headers(self) -> dict:
+        """Create header object for communication packets."""
         header_data = {}
         header_data["Content-Type"] = "application/json"
         header_data["Authorization"] = self._token_type + " " + self._token
 
         return header_data
 
-    def auth(self, username, password, cloud, type="app"):
+    def auth(self):
+        """Authenticate."""
         import json
         import uuid
 
         self.uuid = str(uuid.uuid1())
-        self.cloud = clouds[cloud]
+        self.cloud = clouds[self.type]
         self._api_host = (API_BASE).format(self.cloud["url"])
 
         self._token = self._generate_identify_token(self.cloud["key"])
 
         payload_data = {}
-        payload_data["username"] = username
-        payload_data["password"] = password
+        payload_data["username"] = self.username
+        payload_data["password"] = self.password
         payload_data["grant_type"] = "password"
         payload_data["client_id"] = 1
-        payload_data["type"] = type
+        payload_data["type"] = "app"
         payload_data["client_secret"] = self._token
         payload_data["scope"] = "*"
 
         payload = json.dumps(payload_data)
 
-        callData = self._call("/oauth/token", payload)
+        callData = self._call("/oauth/token", payload, checktoken=False)
 
         return callData
 
     def get_profile(self):
+        """Get user profile."""
         callData = self._call("/users/me")
         self._data = callData
         return callData
 
     def get_cert(self):
+        """Get user certificate."""
         callData = self._call("/users/certificate")
         self._data = callData
         return callData
 
     def get_products(self):
+        """Get devices associated with this user."""
         callData = self._call("/product-items")
         self._data = callData
         return callData
 
     def get_status(self, serial):
+        """Get device status."""
         callStr = "/product-items/{}/status".format(serial)
         callData = self._call(callStr)
         return callData
 
-    def _call(self, path, payload=None):
+    def _call(self, path: str, payload: str = None, checktoken: bool = True):
+        """Do the actual call to the device."""
         import json
 
         import requests
 
+        # Check if token needs refreshing
+        now = int(time.time())  # Current time in unix timestamp format
+        if checktoken and ((self._tokenrefresh + 3000) < now):
+            _LOGGER.debug("Refreshing token")
+            try:
+                auth_data = self.auth()
+
+                if "return_code" in auth_data:
+                    return
+
+                self.set_token(auth_data["access_token"], now)
+                self.set_token_type(auth_data["token_type"])
+            except:
+                _LOGGER.debug("Error occured when refreshing token")
+
         try:
             if payload:
                 req = requests.post(
@@ -168,4 +206,5 @@ class WorxLandroidAPI:
 
     @property
     def data(self):
+        """Return data for device."""
         return self._data
